import pytest
from pydantic_forms.exceptions import FormValidationError

from gso.products import ProductType
from gso.products.product_blocks.site import SiteTier
from gso.products.product_types.site import Site
from gso.services.partners import get_partner_by_name
from gso.services.subscriptions import get_product_id_by_name
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.mark.workflow()
def test_create_site(responses, faker):
    product_id = get_product_id_by_name(ProductType.SITE)
    initial_site_data = [
        {"product": product_id},
        {
            "site_name": faker.site_name(),
            "site_city": faker.city(),
            "site_country": faker.country(),
            "site_country_code": faker.country_code(),
            "site_latitude": "-74.0060",
            "site_longitude": "40.7128",
            "site_bgp_community_id": faker.pyint(),
            "site_internal_id": faker.pyint(),
            "site_tier": SiteTier.TIER1,
            "site_ts_address": faker.ipv4(),
        },
    ]
    result, _, _ = run_workflow("create_site", initial_site_data)
    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Site.from_subscription(subscription_id)
    assert subscription.status == "active"
    assert (
        subscription.description
        == f"Site in {initial_site_data[1]['site_city']}, {initial_site_data[1]['site_country']}"
    )


@pytest.mark.workflow()
def test_site_name_is_incorrect(responses, faker):
    """Test validate site name on site creation.

    The site name is a string with 3 upper case letter and one digit.
    Like: AMS, LON, LON1...LON 9.
    This test checks an invalid string for site name.
    The validation should throw an exception in case of an invalid string.
    """
    invalid_site_name = "AMST10"
    expected_exception_msg = rf".*Enter a valid site name.+Received: {invalid_site_name}.*"
    product_id = get_product_id_by_name(ProductType.SITE)
    initial_site_data = [
        {"product": product_id},
        {
            "site_name": invalid_site_name,
            "site_city": faker.city(),
            "site_country": faker.country(),
            "site_country_code": faker.country_code(),
            "site_latitude": "-74.0060",
            "site_longitude": "40.7128",
            "site_bgp_community_id": faker.pyint(),
            "site_internal_id": faker.pyint(),
            "site_tier": SiteTier.TIER1,
            "site_ts_address": faker.ipv4(),
            "partner": get_partner_by_name("GEANT")["partner_id"],
        },
    ]

    with pytest.raises(FormValidationError, match=expected_exception_msg):
        run_workflow("create_site", initial_site_data)
